/*
 * mpcut - X11 mp3 frame editor
 *
 * Copyright (c) 2000, Warren Toomey, wkt@cs.adfa.edu.au
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Deletions code. Version 1.1b, $Revision: 1.4 $
 */

#include <unistd.h>
#include <stdlib.h>

/* Linked list of deleted selections, kept in inserted order.
 * We DO NOT PREVENT deleted selections in the list from overlapping.
 */
typedef struct delnode {
	int start;
	int end;
	struct delnode *next;
} Delnode;

static Delnode *Delhead=NULL, *this=NULL;

/* Return 1 if the frame falls within a deleted selection,
 * or 0 if the frame is not deleted. We use the cached
 * this pointer to help speed things up.
 */
int is_frame_deleted(int frame)
{
					/* Within the cached selection */
  if ((this!=NULL) && (frame>=this->start) && (frame<=this->end)) return(1);

  if (Delhead==NULL) return(0);		/* No deleted selections */

					/* Otherwise, walk the list */
  for (this=Delhead;this!=NULL;this=this->next) {
    if ((frame>=this->start) && (frame<=this->end)) return(1);
  }
					/* Nope, not in any deleted selection */
  this=NULL; return(0);
}

void clear_deletions(void)
{
  Delnode *t;

  if (Delhead==NULL) return;		/* No deleted selections */

  for (this=Delhead;this!=NULL;this=t) { t=this->next; free(this); }
  Delhead=this=NULL;
}

void insert_deletion(int start, int end)
 {
  Delnode *new;
  int temp;

					/* Ensure start <= end */
  if (end < start) { temp=end; end=start; start=end; }

  new=(Delnode *)malloc(sizeof(Delnode));
  if (new==NULL) return;
  new->start= start; new->end=end;

  new->next= Delhead;
  Delhead=new; return;
 }

/* Remove the first cut in the list, i.e the
 * last cut performed.
 */
void remove_first_deletion(void)
{
  Delnode *t;

  if (Delhead==NULL) return;
  t=Delhead; Delhead=t->next;
  if (this==t) this=NULL;
  free(t);
}
